@@ -2,9 +2,9 @@
 #'
 #' This function is a wrapper for the pathview package (Luo \emph{et al.}, 2017. \emph{Nucleic acids research}, 45:W501-W508). It will generate annotated KEGG pathway maps showing which reactions are present in the different samples. It will also generate legends with the color scales for each sample in separate png files. 
 #'
-#' @param SQM A SQM or SQMlite object.
+#' @param SQM A SQM, SQMbunch or SQMlite object.
 #' @param pathway_id character. The five-number KEGG pathway identifier. A list of all pathway identifiers can be found in \url{https://www.genome.jp/kegg/pathway.html}.
-#' @param count character. Either \code{"abund"} for raw abundances, \code{"percent"} for percentages, \code{"bases"} for raw base counts, \code{"tpm"} for TPM normalized values or \code{"copy_number"} for copy numbers (default \code{"tpm"}). Note that a given count type might not available in this object (e.g. TPM or copy number in SQMlite objects originating from a SQM reads project).
+#' @param count character. Either \code{"abund"} for raw abundances, \code{"percent"} for percentages, \code{"bases"} for raw base counts, \code{"tpm"} for TPM normalized values or \code{"copy_number"} for copy numbers (default \code{"copy_number"}). Note that a given count type might not available in this object (e.g. TPM or copy number in SQMlite objects originating from a SQM reads project).
 #' @param samples character. An optional vector with the names of the samples to export. If absent, all samples will be exported (default \code{NULL}).
 #' @param split_samples logical. Generate a different output file for each sample (default \code{FALSE}).
 #' @param sample_colors character. An optional vector with the plotting colors for each sample (default \code{NULL}).
@@ -14,24 +14,29 @@
 #' @param max_scale_value numeric. Maximum value to include in the color scale. By default it is the maximum value in the selected samples (if plotting abundances in samples) or the maximum absolute log2 fold-change (if plotting fold changes) (default \code{NULL}).
 #' @param color_bins numeric. Number of bins used to generate the gradient in the color scale (default \code{10}).
 #' @param output_suffix character. Suffix to be added to the output files (default \code{"pathview"}).
+#' @param output_dir character. Directory in which to write the output files (default \code{"."}).
 #' @return No return value, but Pathview figures are produced in the current working directory.
 #' @seealso \code{\link{plotFunctions}} for plotting the most functions taxa of a SQM object.
 #' @examples
 #' \donttest{
 #' data(Hadza)
-#' exportPathway(Hadza, "00910", count = 'copy_number', 
-#'               output_suffix = "nitrogen_metabolism", sample_colors = c("red", "blue"))
-#' exportPathway(Hadza, "00250", count = 'tpm', 
+#' 
+#' exportPathway(Hadza, "00910", count = 'copy_number',
+#'               output_dir = tempdir(),
+#'               output_suffix = "nitrogen_metabolism",
+#'               sample_colors = c("red", "blue"))
+#' exportPathway(Hadza, "00250", count = 'tpm',
+#'               output_dir = tempdir(),
 #'               output_suffix = "ala_asp_glu_metabolism_FoldChange", 
 #'               fold_change_groups = list(c("H1"), c("H12")), max_scale_value=2)
 #'}
 #' @importFrom graphics plot rasterImage text
 #' @importFrom grDevices colorRampPalette dev.off png
 #' @export
-exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_samples = FALSE, sample_colors = NULL, log_scale = FALSE, fold_change_groups = NULL, fold_change_colors = NULL, max_scale_value = NULL, color_bins = 10, output_suffix = 'pathview')
+exportPathway = function(SQM, pathway_id, count = 'copy_number', samples = NULL, split_samples = FALSE, sample_colors = NULL, log_scale = FALSE, fold_change_groups = NULL, fold_change_colors = NULL, max_scale_value = NULL, color_bins = 10, output_dir = '.', output_suffix = 'pathview')
     {
     ### Check params.
-    if(!inherits(SQM, c('SQM', 'SQMlite'))) { stop('The first argument must be a SQM or a SQMlite object') }
+    if(!inherits(SQM, c('SQM', 'SQMbunch', 'SQMlite'))) { stop('The first argument must be a SQM or a SQMlite object') }
     if(!count %in% c('abund', 'percent', 'bases', 'tpm', 'copy_number'))
         {
         stop('count must be "abund", "percent", "bases", "tpm" or "copy_number"')
@@ -46,7 +51,8 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
         }
     if( count == 'copy_number' & is.null(SQM[['functions']][['KEGG']][[count]]) )
         {
-        stop('There are no copy number tables in your project, possibly because RecA was not present in the metagenome or it was generated by sqm_reads.pl')
+        warning('There are no copy number tables in your project, possibly because the single copy genes were not present in the metagenome or the project was generated by sqm_reads.pl. Will use "count=\'percent\' instead".')
+        count = 'percent'
         }
     if(!is.null(fold_change_groups))
         {
@@ -86,8 +92,8 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
 
     # Taken from https://support.bioconductor.org/p/58110/
     # Download and parse pathway info.
-    pathview::download.kegg(pathway.id = pathway_id, species = 'ko')
-    xml.file = sprintf('ko%s.xml', pathway_id)
+    pathview::download.kegg(pathway.id = pathway_id, species = 'ko', kegg.dir = output_dir)
+    xml.file = sprintf('%s/ko%s.xml', output_dir, pathway_id)
     node.data = pathview::node.info(xml.file)
     # Map our data.
     plot.data.gene = pathview::node.map(mol.data=mat, node.data, node.types="ortholog", entrez.gnodes=FALSE)
@@ -151,7 +157,7 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
             true_breaks = breaks # Breaks for the legend text
             }
         cols.ts.gene[,i] = gradient[submat_color[,i]]
-        filename = sprintf('ko%s.%s.%s.legend.png', pathway_id, output_suffix, colnames(cols.ts.gene)[i])
+        filename = sprintf('%s/ko%s.%s.%s.legend.png', output_dir, pathway_id, output_suffix, colnames(cols.ts.gene)[i])
         message(sprintf('Info: Writing legend file %s\n', filename))
         png(filename)
         plot(c(0,2),c(0,1),type = 'n', axes = FALSE, xlab = '', ylab = '', main = sprintf('%s - %s', colnames(cols.ts.gene)[i], nice_label[count]))
@@ -162,14 +168,17 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
     cols.ts.gene[zeros] = bg.col # In log2FC plots, if we have an even number of color bins, rxns with zero FC (i.e. absent rxns) will not be exactly white.
 
     ### PLOT
+    thiswd = getwd()
+    on.exit(setwd(thiswd))
+    setwd(output_dir)
     # Are reactions represented as arrows in this map?? If so, keggview.native may fail
     if('line' %in% unique(node.data$shape[node.data$type=='ortholog']))
         {
         warning('This map uses lines to represent reactions, and we\'ve had isues using keggview native plot in this case.
                  We will switch to graph mode instead')
 	parseKGML2Graph2 = utils::getFromNamespace("parseKGML2Graph2", "pathview") 
-        gR1 = parseKGML2Graph2(xml.file, genes = F, 
-                               expand = F, split.group = F)
+        gR1 = parseKGML2Graph2(xml.file, genes = FALSE, 
+                               expand = FALSE, split.group = FALSE)
         plot.data.cpd=pathview::node.map(NULL, node.data, node.types="compound")
 	plot.data.cpd$labels=pathview::cpdkegg2name(plot.data.cpd$labels)[,2]
 	mapped.cnodes=rownames(plot.data.cpd)
@@ -177,6 +186,7 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
         pathview::keggview.graph(plot.data.gene = plot.data.gene,
                                  cols.ts.gene = cols.ts.gene, node.data=node.data,
                                  pathway.name = sprintf('ko%s', pathway_id),
+                                 kegg.dir = output_dir,
 				 path.graph = gR1, map.cpdname = TRUE,
 				 cex = 0.15,
                                  same.layer = TRUE, plot.col.key = FALSE, multi.state=!split_samples, out.suffix = output_suffix)
@@ -184,15 +194,11 @@ exportPathway = function(SQM, pathway_id, count = 'tpm', samples = NULL, split_s
         pathview::keggview.native(plot.data.gene = plot.data.gene,
                                  cols.ts.gene = cols.ts.gene, node.data=node.data,
                                  pathway.name = sprintf('ko%s', pathway_id),
+                                 kegg.dir = output_dir,
                                  same.layer = TRUE, plot.col.key = FALSE, multi.state=!split_samples, out.suffix = output_suffix)
         }
-    system(sprintf('rm ko%s.png ko%s.xml', pathway_id, pathway_id))
+    file.remove(sprintf('%s/ko%s.png', output_dir, pathway_id))
+    file.remove(sprintf('%s/ko%s.xml', output_dir, pathway_id))
+    return(invisible(NULL)) 
     }
 
-
-
-
-
-
-
-
