@@ -157,12 +157,13 @@ plotBars = function(data, label_x = 'Samples', label_y = 'Abundances', label_fil
 #' Heatmap of the most abundant functions in a SQM object
 #'
 #' This function selects the most abundant functions across all samples in a SQM object and represents their abundances in a heatmap. Alternatively, a custom set of functions can be represented.
-#' @param SQM A SQM or SQMlite object.
+#' @param SQM A SQM, SQMbunch or SQMlite object.
 #' @param fun_level character. Either \code{"KEGG"}, \code{"COG"}, \code{"PFAM"} or any other custom database used for annotation (default \code{"KEGG"}).
 #' @param count character. Either \code{"abund"} for raw abundances, \code{"percent"} for percentages, \code{"bases"} for raw base counts, \code{"cpm"} for coverages per million reads, \code{"tpm"} for TPM normalized values or \code{"copy_number"} for copy numbers (default \code{"tpm"}). Note that a given count type might not available in this object (e.g. TPM or copy number in SQMlite objects originating from a SQM reads project).
 #' @param N integer Plot the \code{N} most abundant functions (default \code{25}).
 #' @param fun character. Custom functions to plot. If provided, it will override \code{N} (default \code{NULL}).
 #' @param samples character. Character vector with the names of the samples to include in the plot. Can also be used to plot the samples in a custom order. If not provided, all samples will be plotted (default \code{NULL}).
+#' @param display_function_names logical. Plot function names alongside function IDs, if available (default \code{TRUE}).
 #' @param ignore_unmapped logical. Don't include unmapped reads in the plot (default \code{TRUE}).
 #' @param ignore_unclassified logical. Don't include unclassified ORFs in the plot (default \code{TRUE}).
 #' @param gradient_col A vector of two colors representing the low and high ends of the color gradient (default \code{c("ghostwhite", "dodgerblue4")}).
@@ -174,9 +175,9 @@ plotBars = function(data, label_x = 'Samples', label_y = 'Abundances', label_fil
 #' data(Hadza)
 #' plotFunctions(Hadza)
 #' @export
-plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = NULL, samples = NULL, ignore_unmapped = TRUE, ignore_unclassified = TRUE, gradient_col = c('ghostwhite', 'dodgerblue4'), base_size = 11, metadata_groups = NULL)
+plotFunctions = function(SQM, fun_level = 'KEGG', count = 'copy_number', N = 25, fun = NULL, samples = NULL, display_function_names = TRUE, ignore_unmapped = TRUE, ignore_unclassified = TRUE, gradient_col = c('ghostwhite', 'dodgerblue4'), base_size = 11, metadata_groups = NULL)
     {
-    if(!inherits(SQM, c('SQM', 'SQMlite'))) { stop('The first argument must be a SQM or a SQMlite object') }
+    if(!inherits(SQM, c('SQM', 'SQMbunch', 'SQMlite'))) { stop('The first argument must be a SQM, SQMbunch or a SQMlite object') }
     if (!fun_level %in% names(SQM$functions))
         {
         intro = 'Select function category among'
@@ -202,7 +203,8 @@ plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = N
         }
     if( count == 'copy_number' & is.null(SQM[['functions']][[fun_level]][[count]]) )
         {
-        stop('There are no copy number tables in your project, possibly because RecA was not present in the metagenome or it was generated by sqm_reads.pl. Try with "count=\'percent\'".')
+        warning('There are no copy number tables in your project, possibly because the single copy genes were not present in the metagenome or the project was generated by sqm_reads.pl. Will use "count=\'percent\' instead".')
+        count = 'percent'
         }
     if (is.null(fun) & N <= 0)
         {
@@ -233,8 +235,9 @@ plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = N
 
     data = mostAbundant(data, N = N, items = fun)
 
-    # Add KEGG & COG names
-    if (fun_level %in% c('COG', 'KEGG'))
+    # Add KEGG & COG names, also extnames if available
+    names_string = paste0(fun_level, '_names')
+    if (names_string %in% names(SQM$misc) & display_function_names)
         {
         item_name = SQM$misc[[paste0(fun_level, '_names')]][rownames(data)]
         item_name[is.na(item_name)] = 'no_name'
@@ -242,6 +245,7 @@ plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = N
         item_name = sapply(item_name, FUN = function(x) paste(strwrap(x, width = 50), collapse = '\n'))
         rownames(data) = item_name
         }
+
     # Put unclassified and unmapped at the bottom
     if('Unclassified' %in% rownames(data)) # Put unclassified at the bottom.
         {
@@ -272,7 +276,7 @@ plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = N
 #' Barplot of the most abundant taxa in a SQM object
 #'
 #' This function selects the most abundant taxa across all samples in a SQM object and represents their abundances in a barplot. Alternatively, a custom set of taxa can be represented.
-#' @param SQM A SQM or a SQMlite object.
+#' @param SQM A SQM, SQMbunch or a SQMlite object.
 #' @param rank Taxonomic rank to plot (default \code{phylum}).
 #' @param count character. Either \code{"percent"} for percentages, or \code{"abund"} for raw abundances (default \code{"percent"}).
 #' @param N integer Plot the \code{N} most abundant taxa (default \code{15}).
@@ -298,7 +302,7 @@ plotFunctions = function(SQM, fun_level = 'KEGG', count = 'tpm', N = 25, fun = N
 #' @export
 plotTaxonomy = function(SQM, rank = 'phylum', count = 'percent', N = 15, tax = NULL, others = TRUE, samples = NULL, nocds = 'treat_separately', ignore_unmapped = FALSE, ignore_unclassified = FALSE, no_partial_classifications = FALSE, rescale = FALSE, color = NULL, base_size = 11, max_scale_value = NULL, metadata_groups = NULL)
     {
-    if(!inherits(SQM, c('SQM', 'SQMlite'))) { stop('The first argument must be a SQM or a SQMlite object') }
+    if(!inherits(SQM, c('SQM', 'SQMbunch', 'SQMlite'))) { stop('The first argument must be a SQM, SQMbunch, or a SQMlite object') }
     if (!rank %in% c('superkingdom', 'phylum', 'class', 'order', 'family', 'genus', 'species'))
         {
         stop('Select rank among "superkingdom", "phylum", "class", "order", "family", "genus" or "species". and count between \'percent\' or \'abund\'')
@@ -474,7 +478,7 @@ plotTaxonomy = function(SQM, rank = 'phylum', count = 'percent', N = 15, tax = N
 #' Barplot of the most abundant bins in a SQM object
 #'
 #' This function selects the most abundant bins across all samples in a SQM object and represents their abundances in a barplot. Alternatively, a custom set of bins can be represented.
-#' @param SQM A SQM or a SQMlite object.
+#' @param SQM A SQM object.
 #' @param count character. Either \code{"abund"} for raw abundances, \code{"percent"} for percentages, \code{"cov"} for coverages, or \code{"cpm"} for coverages per million reads (default \code{"percent"}).
 #' @param N integer Plot the \code{N} most abundant bins (default \code{15}).
 #' @param bins character. Custom bins to plot. If provided, it will override \code{N} (default \code{NULL}).
@@ -488,7 +492,7 @@ plotTaxonomy = function(SQM, rank = 'phylum', count = 'percent', N = 15, tax = N
 #' @param max_scale_value numeric. Maximum value to include in the y axis. By default it is handled automatically by ggplot2 (default \code{NULL}).
 #' @param metadata_groups list. Split the plot into groups defined by the user: list('G1' = c('sample1', sample2'), 'G2' = c('sample3', 'sample4')) default \code{NULL}).
 #' @return a ggplot2 plot object.
-#' @seealso \code{\link{plotBins}} for plotting the most abundant bins of a SQM object; \code{\link{plotBars}} and \code{\link{plotHeatmap}} for plotting barplots or heatmaps with arbitrary data.
+#' @seealso \code{\link{plotTaxonomy}} for plotting the most abundant taxa of a SQM object; \code{\link{plotBars}} and \code{\link{plotHeatmap}} for plotting barplots or heatmaps with arbitrary data.
 #' @examples
 #' data(Hadza)
 #' # Bins distribution.
@@ -496,11 +500,15 @@ plotTaxonomy = function(SQM, rank = 'phylum', count = 'percent', N = 15, tax = N
 #' @export
 plotBins = function(SQM, count = 'percent', N = 15, bins = NULL, others = TRUE, samples = NULL, ignore_unmapped = FALSE, ignore_nobin = FALSE, rescale = FALSE, color = NULL, base_size = 11, max_scale_value = NULL, metadata_groups = NULL)
 {
-    if(!inherits(SQM, c('SQM', 'SQMlite'))) { stop('The first argument must be a SQM or a SQMlite object') }
+    if(!inherits(SQM, c('SQM'))) { stop('The first argument must be a SQM object') }
     if (!count %in% c('abund', 'percent', 'cov', 'cpm'))
     {
         stop('count must be either "abund", "percent", "cov", "cpm"')
     }
+    if(is.null(SQM[['bins']]))
+    {
+        stop('This project contains no bins!')
+    }
     if ('Other' %in% rownames(SQM[['bins']][[count]]))
     {
         stop('One of your bin is called "Other", please change its name')
