@@ -1,3 +1,24 @@
+subsetDispatch = function(f, SQM, ...)
+    {
+    if(!inherits(SQM, c('SQM', 'SQMbunch'))) { stop('The first argument must be a SQM or SQMbunch object') }
+    if(!inherits(SQM, 'SQMbunch'))
+        {
+        args = c(list(SQM=SQM), list(...))
+        subSQM = do.call(f, args)
+    } else {
+        projs = list()
+        for(p in SQM$projects)
+            {
+            args = c(list(SQM=p), list(...))
+	    args$allow_empty = TRUE
+            projs = c(projs, list(do.call(f, args)))
+            }
+        subSQM = combineSQM(projs)
+        }
+    return(subSQM) 
+    }
+
+
 #' Filter results by sample
 #'
 #' Create a SQM object containing only samples specified by the user, and the ORFs, contigs, bins, taxa and functions present in those samples.
@@ -15,22 +36,32 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
     subSQM$misc$samples = samples
     subSQM$total_reads = subSQM$total_reads[samples]
     
-    # ORFs
+    ### ORFs
     if(remove_missing) {
         presentORFs = rownames(subSQM$orfs$abund)[ rowSums(subSQM$orfs$abund[,samples,drop=FALSE]) > 0 ]
     } else {
         presentORFs = rownames(subSQM$orfs$abund)
     }
-    subSQM$orfs$table = subSQM$orfs$table[intersect(presentORFs, rownames(subSQM$orfs$table)),       ,drop=FALSE]
-    subSQM$orfs$abund = subSQM$orfs$abund[intersect(presentORFs, rownames(subSQM$orfs$abund)),samples,drop=FALSE]
-    subSQM$orfs$bases = subSQM$orfs$bases[intersect(presentORFs, rownames(subSQM$orfs$bases)),samples,drop=FALSE]
-    subSQM$orfs$cov   = subSQM$orfs$cov  [intersect(presentORFs, rownames(subSQM$orfs$cov  )),samples,drop=FALSE]
-    subSQM$orfs$cpm   = subSQM$orfs$cpm  [intersect(presentORFs, rownames(subSQM$orfs$cpm  )),samples,drop=FALSE]
-    subSQM$orfs$tpm   = subSQM$orfs$tpm  [intersect(presentORFs, rownames(subSQM$orfs$tpm  )),samples,drop=FALSE]
-    subSQM$orfs$seqs  = subSQM$orfs$seqs [intersect(presentORFs,    names(subSQM$orfs$seqs )),        drop=FALSE]
-    subSQM$orfs$tax   = subSQM$orfs$tax  [intersect(presentORFs, rownames(subSQM$orfs$tax  )),       ,drop=FALSE]
-
-    # Contigs
+    subSQM$orfs$table       = subSQM$orfs$table[intersect(presentORFs, rownames(subSQM$orfs$table )),       ,drop=FALSE]
+    subSQM$orfs$abund       = subSQM$orfs$abund[intersect(presentORFs, rownames(subSQM$orfs$abund )),samples,drop=FALSE]
+    subSQM$orfs$bases       = subSQM$orfs$bases[intersect(presentORFs, rownames(subSQM$orfs$bases )),samples,drop=FALSE]
+    subSQM$orfs$cov         = subSQM$orfs$cov  [intersect(presentORFs, rownames(subSQM$orfs$cov   )),samples,drop=FALSE]
+    subSQM$orfs$cpm         = subSQM$orfs$cpm  [intersect(presentORFs, rownames(subSQM$orfs$cpm   )),samples,drop=FALSE]
+    subSQM$orfs$tpm         = subSQM$orfs$tpm  [intersect(presentORFs, rownames(subSQM$orfs$tpm   )),samples,drop=FALSE]
+    if(!is.null(subSQM$orfs$seqs))
+        {        
+        subSQM$orfs$seqs    = subSQM$orfs$seqs [intersect(presentORFs,    names(subSQM$orfs$seqs  )),        drop=FALSE]
+        }
+    subSQM$orfs$tax         = subSQM$orfs$tax  [intersect(presentORFs, rownames(subSQM$orfs$tax   )),       ,drop=FALSE]
+    if('tax16S' %in% names(subSQM$orfs))
+        {
+        subSQM$orfs$tax16   = SQM$orfs$tax16S  [intersect(presentORFs, rownames(subSQM$orfs$tax16S)),       ,drop=FALSE]
+        }
+    if('markers' %in% names(subSQM$orfs))
+        {
+        subSQM$orfs$markers = SQM$orfs$markers [intersect(presentORFs, names(subSQM$orfs$markers  )),        drop=FALSE]
+        }
+    ### Contigs
     if(remove_missing) {
         presentContigs = rownames(subSQM$contigs$abund)[ rowSums(subSQM$contigs$abund[,samples,drop=FALSE]) > 0 ]
     } else {
@@ -42,11 +73,14 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
     subSQM$contigs$cov   = subSQM$contigs$cov  [intersect(presentContigs, rownames(subSQM$contigs$cov  )),samples,drop=FALSE]
     subSQM$contigs$cpm   = subSQM$contigs$cpm  [intersect(presentContigs, rownames(subSQM$contigs$cpm  )),samples,drop=FALSE]
     subSQM$contigs$tpm   = subSQM$contigs$tpm  [intersect(presentContigs, rownames(subSQM$contigs$tpm  )),samples,drop=FALSE]
-    subSQM$contigs$seqs  = subSQM$contigs$seqs [intersect(presentContigs, rownames(subSQM$contigs$seqs )),        drop=FALSE]
+    if(!is.null(subSQM$contigs$seqs))
+        {
+        subSQM$contigs$seqs  = subSQM$contigs$seqs [intersect(presentContigs, rownames(subSQM$contigs$seqs )),        drop=FALSE]
+        }
     subSQM$contigs$tax   = subSQM$contigs$tax  [intersect(presentContigs, rownames(subSQM$contigs$tax  )),       ,drop=FALSE]
     subSQM$contigs$bins  = subSQM$contigs$bins [intersect(presentContigs, rownames(subSQM$contigs$bins )),       ,drop=FALSE]
     
-    # Bins
+    ### Bins
     if('bins' %in% names(subSQM)) {
         if(remove_missing) {
             presentBins = rownames(subSQM$bins$abund)[ rowSums(subSQM$bins$abund[,samples,drop=FALSE]) > 0 ]
@@ -63,7 +97,7 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
         subSQM$bins$tax     = subSQM$bins$tax    [intersect(presentBins, rownames(subSQM$bins$tax    )),       ,drop=FALSE]
     }
 
-    # Taxa
+    ### Taxa
     for(rank in names(subSQM$taxa)) {
         if(remove_missing) {
             presentTaxa = rownames(subSQM$taxa[[rank]]$abund)[ rowSums(subSQM$taxa[[rank]]$abund[,samples,drop=FALSE]) > 0 ]
@@ -76,7 +110,7 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
 	}
     }
 
-    # Functions
+    ### Functions
     for(method in names(subSQM$functions)) {
         if(remove_missing) {
             presentFuns = rownames(subSQM$functions[[method]]$abund)[ rowSums(subSQM$functions[[method]]$abund[,samples,drop=FALSE]) > 0 ]
@@ -94,8 +128,8 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
 
 #' Filter results by function
 #'
-#' Create a SQM object containing only the ORFs with a given function, and the contigs and bins that contain them.
-#' @param SQM SQM object to be subsetted.
+#' Create a SQM or SQMbunch object containing only the ORFs with a given function, and the contigs and bins that contain them.
+#' @param SQM SQM or SQMbunch object to be subsetted.
 #' @param fun character. Pattern to search for in the different functional classifications.
 #' @param columns character. Restrict the search to the provided column names from \code{SQM$orfs$table}. If not provided the search will be performed in all the columns containing functional information (default \code{NULL}).
 #' @param ignore_case logical Make pattern matching case-insensitive (default \code{TRUE}).
@@ -104,8 +138,10 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
 #' @param ignore_unclassified_functions logical. If \code{FALSE}, ORFs with no functional classification will be aggregated together into an "Unclassified" category. If \code{TRUE}, they will be ignored (default \code{FALSE}).
 #' @param rescale_tpm logical. If \code{TRUE}, TPMs for KEGGs, COGs, and PFAMs will be recalculated (so that the TPMs in the subset actually add up to 1 million). Otherwise, per-function TPMs will be calculated by aggregating the TPMs of the ORFs annotated with that function, and will thus keep the scaling present in the parent object (default \code{FALSE}).
 #' @param rescale_copy_number logical. If \code{TRUE}, copy numbers with be recalculated using the RecA/RadA coverages in the subset. Otherwise, RecA/RadA coverages will be taken from the parent object. By default it is set to \code{FALSE}, which means that the returned copy numbers for each function will represent the average copy number of that function per genome in the parent object.
+#' @param recalculate_bin_stats logical. If \code{TRUE}, bin stats and taxonomy are recalculated based on the contigs present in the subsetted object (default \code{FALSE}).
+#' @param allow_empty (internal use only).
 #' @seealso \code{\link{subsetTax}}, \code{\link{subsetORFs}}, \code{\link{subsetSamples}}, \code{\link{combineSQM}}. The most abundant items of a particular table contained in a SQM object can be selected with \code{\link{mostAbundant}}.
-#' @return SQM object containing only the requested function.
+#' @return SQM or SQMbunch object containing only the requested function.
 #' @examples
 #' data(Hadza)
 #' Hadza.iron = subsetFun(Hadza, "iron")
@@ -113,7 +149,19 @@ subsetSamples = function(SQM, samples, remove_missing = TRUE)
 #' # Search for multiple patterns using regular expressions
 #' Hadza.twoKOs = subsetFun(Hadza, "K00812|K00813", fixed=FALSE)
 #' @export
-subsetFun = function(SQM, fun, columns = NULL, ignore_case=TRUE, fixed=FALSE, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = FALSE, rescale_copy_number = FALSE)
+subsetFun = function(SQM, fun, columns = NULL, ignore_case=TRUE, fixed=FALSE,
+                     trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE,
+                     rescale_tpm = FALSE, rescale_copy_number = FALSE, recalculate_bin_stats = FALSE, allow_empty = FALSE)
+    {
+    return(subsetDispatch(subsetFun_, SQM, fun, columns=columns, ignore_case = ignore_case, fixed = fixed,
+                     trusted_functions_only = trusted_functions_only, ignore_unclassified_functions = ignore_unclassified_functions,
+                     rescale_tpm = rescale_tpm, rescale_copy_number = rescale_copy_number, recalculate_bin_stats = recalculate_bin_stats,
+		     allow_empty = allow_empty)
+          )
+    }
+subsetFun_ = function(SQM, fun, columns, ignore_case, fixed,
+                      trusted_functions_only, ignore_unclassified_functions,
+		      rescale_tpm, rescale_copy_number, recalculate_bin_stats, allow_empty)
     {
     if(!inherits(SQM, 'SQM')) { stop('The first argument must be a SQM object') }
 
@@ -133,7 +181,9 @@ subsetFun = function(SQM, fun, columns = NULL, ignore_case=TRUE, fixed=FALSE, tr
                         trusted_functions_only = trusted_functions_only,
                         ignore_unclassified_functions=ignore_unclassified_functions,
                         rescale_tpm         = rescale_tpm,
-                        rescale_copy_number = rescale_copy_number)
+                        rescale_copy_number = rescale_copy_number,
+                        recalculate_bin_stats = recalculate_bin_stats,
+		        allow_empty = allow_empty)
            )
 
     }
@@ -142,7 +192,7 @@ subsetFun = function(SQM, fun, columns = NULL, ignore_case=TRUE, fixed=FALSE, tr
 
 #' Filter results by taxonomy
 #'
-#' Create a SQM object containing only the contigs with a given consensus taxonomy, the ORFs contained in them and the bins that contain them.
+#' Create a SQM or SQMbunch object containing only the contigs with a given consensus taxonomy, the ORFs contained in them and the bins that contain them.
 #' @param SQM SQM object to be subsetted.
 #' @param rank character. The taxonomic rank from which to select the desired taxa (\code{superkingdom}, \code{phylum}, \code{class}, \code{order}, \code{family}, \code{genus}, \code{species})
 #' @param tax character. The taxon to select.
@@ -150,14 +200,24 @@ subsetFun = function(SQM, fun, columns = NULL, ignore_case=TRUE, fixed=FALSE, tr
 #' @param ignore_unclassified_functions logical. If \code{FALSE}, ORFs with no functional classification will be aggregated together into an "Unclassified" category. If \code{TRUE}, they will be ignored (default \code{FALSE}).
 #' @param rescale_tpm logical. If \code{TRUE}, TPMs for KEGGs, COGs, and PFAMs will be recalculated (so that the TPMs in the subset actually add up to 1 million). Otherwise, per-function TPMs will be calculated by aggregating the TPMs of the ORFs annotated with that function, and will thus keep the scaling present in the parent object. By default it is set to \code{TRUE}, which means that the returned TPMs will be scaled \emph{by million of reads of the selected taxon}.
 #' @param rescale_copy_number logical. If \code{TRUE}, copy numbers with be recalculated using the RecA/RadA coverages in the subset. Otherwise, RecA/RadA coverages will be taken from the parent object. By default it is set to \code{TRUE}, which means that the returned copy numbers for each function will represent the average copy number of that function \emph{per genome of the selected taxon}.
-#' @return SQM object containing only the requested taxon.
+#' @param recalculate_bin_stats logical. If \code{TRUE}, bin stats and taxonomy are recalculated based on the contigs present in the subsetted object (default \code{TRUE}).
+#' @param allow_empty (internal use only).
+#' @return SQM or SQMbunch object containing only the requested taxon.
 #' @seealso \code{\link{subsetFun}}, \code{\link{subsetContigs}}, \code{\link{subsetSamples}}, \code{\link{combineSQM}}. The most abundant items of a particular table contained in a SQM object can be selected with \code{\link{mostAbundant}}.
 #' @examples
 #' data(Hadza)
 #' Hadza.Prevotella = subsetTax(Hadza, "genus", "Prevotella")
-#' Hadza.Proteobacteria = subsetTax(Hadza, "phylum", "Proteobacteria")
+#' Hadza.Bacteroidota = subsetTax(Hadza, "phylum", "Bacteroidota")
 #' @export
-subsetTax = function(SQM, rank, tax, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = TRUE, rescale_copy_number = TRUE)
+subsetTax = function(SQM, rank, tax, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = TRUE, rescale_copy_number = TRUE, recalculate_bin_stats = TRUE, allow_empty = FALSE)
+    {
+    return(subsetDispatch(subsetTax_, SQM, rank, tax,
+			  trusted_functions_only = trusted_functions_only, ignore_unclassified_functions = ignore_unclassified_functions,
+			  rescale_tpm = rescale_tpm, rescale_copy_number = rescale_copy_number, recalculate_bin_stats = recalculate_bin_stats,
+			  allow_empty = allow_empty)
+          )
+    }
+subsetTax_ = function(SQM, rank, tax, trusted_functions_only, ignore_unclassified_functions, rescale_tpm, rescale_copy_number, recalculate_bin_stats, allow_empty)
     {
     if(!inherits(SQM, 'SQM')) { stop('The first argument must be a SQM object') }
     if(!rank %in% colnames(SQM$contigs$tax)) { stop(sprintf('Valid taxonomic ranks are %s', paste(colnames(SQM$contigs$tax), collapse = ', '))) }
@@ -166,7 +226,9 @@ subsetTax = function(SQM, rank, tax, trusted_functions_only = FALSE, ignore_uncl
                            trusted_functions_only = trusted_functions_only,
                            ignore_unclassified_functions=ignore_unclassified_functions,
                            rescale_tpm = rescale_tpm,
-                           rescale_copy_number = rescale_copy_number)
+                           rescale_copy_number = rescale_copy_number,
+	                   recalculate_bin_stats = recalculate_bin_stats,
+	                   allow_empty = allow_empty)
            )
    
     }
@@ -181,6 +243,7 @@ subsetTax = function(SQM, rank, tax, trusted_functions_only = FALSE, ignore_uncl
 #' @param ignore_unclassified_functions logical. If \code{FALSE}, ORFs with no functional classification will be aggregated together into an "Unclassified" category. If \code{TRUE}, they will be ignored (default \code{FALSE}).
 #' @param rescale_tpm logical. If \code{TRUE}, TPMs for KEGGs, COGs, and PFAMs will be recalculated (so that the TPMs in the subset actually add up to 1 million). Otherwise, per-function TPMs will be calculated by aggregating the TPMs of the ORFs annotated with that function, and will thus keep the scaling present in the parent object. By default it is set to \code{TRUE}, which means that the returned TPMs will be scaled \emph{by million of reads of the selected bins}.
 #' @param rescale_copy_number logical. If \code{TRUE}, copy numbers with be recalculated using the RecA/RadA coverages in the subset. Otherwise, RecA/RadA coverages will be taken from the parent object. By default it is set to \code{TRUE}, which means that the returned copy numbers for each function will represent the average copy number of that function \emph{per genome of the selected bins}.
+#' @param allow_empty (internal use only).
 #' @return SQM object containing only the requested bins.
 #' @seealso \code{\link{subsetContigs}}, \code{\link{subsetORFs}}
 #' @examples 
@@ -191,7 +254,14 @@ subsetTax = function(SQM, rank, tax, trusted_functions_only = FALSE, ignore_uncl
 #' # Subset with the most complete bin.
 #' topBin = subsetBins(Hadza, topBinNames[1])
 #' @export
-subsetBins = function(SQM, bins, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = TRUE, rescale_copy_number = TRUE)
+subsetBins = function(SQM, bins, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = TRUE, rescale_copy_number = TRUE, allow_empty = FALSE)
+    {
+    return(subsetDispatch(subsetBins_, SQM, bins,
+			  trusted_functions_only = trusted_functions_only, ignore_unclassified_functions = ignore_unclassified_functions,
+			  rescale_tpm = rescale_tpm, rescale_copy_number = rescale_copy_number, allow_empty = allow_empty)
+          )
+    }
+subsetBins_ = function(SQM, bins, trusted_functions_only, ignore_unclassified_functions, rescale_tpm, rescale_copy_number, allow_empty)
     {
     if(!inherits(SQM, 'SQM')) { stop('The first argument must be a SQM object') }
     goodContigs = rownames(SQM$contigs$bins)[SQM$contigs$bins %in% bins]
@@ -199,7 +269,9 @@ subsetBins = function(SQM, bins, trusted_functions_only = FALSE, ignore_unclassi
                            trusted_functions_only = trusted_functions_only,
                            ignore_unclassified_functions=ignore_unclassified_functions,
                            rescale_tpm = rescale_tpm,
-                           rescale_copy_number = rescale_copy_number)
+                           rescale_copy_number = rescale_copy_number,
+	                   recalculate_bin_stats = FALSE,
+	                   allow_empty = allow_empty)
            )
     }
 
@@ -213,6 +285,8 @@ subsetBins = function(SQM, bins, trusted_functions_only = FALSE, ignore_unclassi
 #' @param ignore_unclassified_functions logical. If \code{FALSE}, ORFs with no functional classification will be aggregated together into an "Unclassified" category. If \code{TRUE}, they will be ignored (default \code{FALSE}).
 #' @param rescale_tpm logical. If \code{TRUE}, TPMs for KEGGs, COGs, and PFAMs will be recalculated (so that the TPMs in the subset actually add up to 1 million). Otherwise, per-function TPMs will be calculated by aggregating the TPMs of the ORFs annotated with that function, and will thus keep the scaling present in the parent object (default \code{FALSE}).
 #' @param rescale_copy_number logical. If \code{TRUE}, copy numbers with be recalculated using the RecA/RadA coverages in the subset. Otherwise, RecA/RadA coverages will be taken from the parent object. By default it is set to \code{FALSE}, which means that the returned copy numbers for each function will represent the average copy number of that function per genome in the parent object.
+#' @param recalculate_bin_stats logical. If \code{TRUE}, bin stats and taxonomy are recalculated based on the contigs present in the subsetted object (default \code{TRUE}).
+#' @param allow_empty (internal use only).
 #' @return SQM object containing only the selected contigs.
 #' @seealso \code{\link{subsetORFs}}
 #' @examples
@@ -222,7 +296,15 @@ subsetBins = function(SQM, bins, trusted_functions_only = FALSE, ignore_unclassi
 #' lowGCcontigs = subsetContigs(Hadza, lowGCcontigNames)
 #' hist(lowGCcontigs$contigs$table[,"GC perc"])
 #' @export
-subsetContigs = function(SQM, contigs, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = FALSE, rescale_copy_number = FALSE)
+subsetContigs = function(SQM, contigs, trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = FALSE, rescale_copy_number = FALSE, recalculate_bin_stats = TRUE, allow_empty = FALSE)
+    {
+    return(subsetDispatch(subsetContigs_, SQM, contigs,
+			  trusted_functions_only = trusted_functions_only, ignore_unclassified_functions = ignore_unclassified_functions,
+			  rescale_tpm = rescale_tpm, rescale_copy_number = rescale_copy_number, recalculate_bin_stats = recalculate_bin_stats,
+			  allow_empty = allow_empty)
+          )
+    }
+subsetContigs_ = function(SQM, contigs, trusted_functions_only, ignore_unclassified_functions, rescale_tpm, rescale_copy_number, recalculate_bin_stats, allow_empty)
     {
     if(!inherits(SQM, 'SQM')) { stop('The first argument must be a SQM object') }
     goodORFs = rownames(SQM$orfs$table)[SQM$orfs$table[,'Contig ID'] %in% contigs]
@@ -231,7 +313,9 @@ subsetContigs = function(SQM, contigs, trusted_functions_only = FALSE, ignore_un
                         ignore_unclassified_functions=ignore_unclassified_functions,
                         rescale_tpm = rescale_tpm,
                         rescale_copy_number = rescale_copy_number,
-	                contigs_override = contigs)
+			recalculate_bin_stats = recalculate_bin_stats,
+	                contigs_override = contigs,
+	                allow_empty = allow_empty)
            )
     }
 
@@ -262,10 +346,12 @@ subsetRand = function(SQM, N)
 #' @param ignore_unclassified_functions logical. If \code{FALSE}, ORFs with no functional classification will be aggregated together into an "Unclassified" category. If \code{TRUE}, they will be ignored (default \code{FALSE}).
 #' @param rescale_tpm logical. If \code{TRUE}, TPMs for KEGGs, COGs, and PFAMs will be recalculated (so that the TPMs in the subset actually add up to 1 million). Otherwise, per-function TPMs will be calculated by aggregating the TPMs of the ORFs annotated with that function, and will thus keep the scaling present in the parent object (default \code{FALSE}).
 #' @param rescale_copy_number logical. If \code{TRUE}, copy numbers with be recalculated using the RecA/RadA coverages in the subset. Otherwise, RecA/RadA coverages will be taken from the parent object. By default it is set to \code{FALSE}, which means that the returned copy numbers for each function will represent the average copy number of that function per genome in the parent object.
+#' @param recalculate_bin_stats logical. If \code{TRUE}, bin stats and taxonomy are recalculated based on the contigs present in the subsetted object (default \code{TRUE}).
 #' @param contigs_override character. Optional vector of contigs to be included in the subsetted object.
+#' @param allow_empty (internal use only).
 #' @return SQM object containing the requested ORFs.
 #' @section A note on contig/bins subsetting:
-#' While this function selects the contigs and bins that contain the desired orfs, it DOES NOT recalculate contig/bin abundance and statistics based on the selected ORFs only. This means that the abundances presented in tables such as \code{SQM$contig$abund} or \code{SQM$bins$tpm} will still refer to the complete contigs and bins, regardless of whether only a fraction of their ORFs are actually present in the returned SQM object. This is also true for the statistics presented in \code{SQM$contigs$table} and \code{SQM$bins$table}.
+#' While this function selects the contigs and bins that contain the desired orfs, it DOES NOT recalculate contig abundance and statistics based on the selected ORFs only. This means that the abundances presented in tables such as \code{SQM$contig$abund} will still refer to the complete contigs, regardless of whether only a fraction of their ORFs are actually present in the returned SQM object. This is also true for the statistics presented in \code{SQM$contigs$table}. Bin statistics may be recalculated if \code{rescale_copy_number} is set to \code{TRUE}, but recalculation will be based on contigs, not ORFs.
 #' @examples
 #' data(Hadza)
 #' # Select the 100 most abundant ORFs in our dataset.
@@ -273,13 +359,55 @@ subsetRand = function(SQM, N)
 #' mostAbundantORFs = subsetORFs(Hadza, mostAbundantORFnames)
 #' @importFrom stats aggregate
 #' @export
-subsetORFs = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, rescale_tpm = FALSE, rescale_copy_number = FALSE, contigs_override = NULL)
+subsetORFs = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE,
+                      rescale_tpm = FALSE, rescale_copy_number = FALSE, recalculate_bin_stats = TRUE,
+                      contigs_override = NULL, allow_empty = FALSE)
+    {
+    return(subsetDispatch(subsetORFs_, SQM, orfs, tax_source = tax_source,
+			  trusted_functions_only = trusted_functions_only, ignore_unclassified_functions = ignore_unclassified_functions,
+                          rescale_tpm = rescale_tpm, rescale_copy_number = rescale_copy_number, recalculate_bin_stats = recalculate_bin_stats,
+			  contigs_override = contigs_override, allow_empty = allow_empty)
+          ) 
+    }
+subsetORFs_ = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = FALSE, ignore_unclassified_functions = FALSE, 
+		      rescale_tpm = FALSE, rescale_copy_number = FALSE, recalculate_bin_stats = TRUE,
+		      contigs_override = NULL, allow_empty = FALSE)
     {
 
     if(!inherits(SQM, 'SQM')) { stop('The first argument must be a SQM object') }
-    if(length(orfs)==0) { stop('No ORFs were selected. Perhaps the subset query yielded no results?') }
     if(!tax_source %in% c('contigs', 'orfs')) { stop('tax_source must be "orfs" or "contigs"') }
-   
+
+    ### If we got no ORFs, either fail or return an empty object
+    if(length(orfs)==0)
+        {
+        if(!allow_empty) {
+            stop('No ORFs were selected. Perhaps the subset query yielded no results?')
+	} else 
+            {
+            subSQM = SQM
+	    subSQM$orfs = NULL
+	    subSQM$contigs = NULL
+	    subSQM$bins = NULL
+	    for(rank in names(subSQM$taxa))
+                {
+                for(count in names(subSQM$taxa[[rank]]))
+                    {
+                    subSQM$taxa[[rank]][[count]] = subSQM$taxa[[rank]][[count]][0,,drop=F]
+                    }
+                }
+            for(method in names(subSQM$functions))
+                {
+                for(count in names(subSQM$functions[[method]]))
+                    {
+                    subSQM$functions[[method]][[count]] = subSQM$functions[[method]][[count]][0,,drop=F]
+                    }
+                }
+            subSQM$misc = list(project_name = subSQM$misc$project_name, samples = subSQM$misc$samples, ext_annot_sources = subSQM$misc$ext_annot_sources) 
+            return(subSQM)
+	    }
+        }
+
+    ### Now go for it 
     orfs    = rownames(SQM$orfs$table[orfs,,drop=FALSE]) # Make sure it will work if orfs is a bool vector too.
     if(is.null(contigs_override)) { contigs = unique(SQM$orfs$table[orfs,'Contig ID'])
     } else { contigs = contigs_override } # so we can include contigs without ORFs if required
@@ -287,50 +415,79 @@ subsetORFs = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = F
     bins    = bins[bins!='No_bin']
     
     subSQM = SQM
+    ### ORFs
+    #    Table
+    subSQM$orfs$table                 = SQM$orfs$table  [orfs    ,,drop=FALSE]
+    #    Abundances
+    subSQM$orfs$abund                 = SQM$orfs$abund  [orfs    ,,drop=FALSE]
 
-    subSQM$orfs$table                 = SQM$orfs$table[orfs      ,,drop=FALSE]
-    subSQM$orfs$abund                 = SQM$orfs$abund[orfs      ,,drop=FALSE]
-    subSQM$orfs$bases                 = SQM$orfs$bases[orfs      ,,drop=FALSE]
-    subSQM$orfs$cov                   = SQM$orfs$cov[orfs        ,,drop=FALSE]
-    subSQM$orfs$cpm                   = SQM$orfs$cpm[orfs        ,,drop=FALSE]
-    subSQM$orfs$tpm                   = SQM$orfs$tpm[orfs        ,,drop=FALSE]
-    subSQM$orfs$seqs                  = SQM$orfs$seqs[orfs]
-    subSQM$orfs$tax                   = SQM$orfs$tax[orfs        ,,drop=FALSE]
-
+    subSQM$orfs$bases                 = SQM$orfs$bases  [orfs    ,,drop=FALSE]
+    subSQM$orfs$cov                   = SQM$orfs$cov    [orfs    ,,drop=FALSE]
+    subSQM$orfs$cpm                   = SQM$orfs$cpm    [orfs    ,,drop=FALSE]
+    subSQM$orfs$tpm                   = SQM$orfs$tpm    [orfs    ,,drop=FALSE]
+    #    Sequences
+    if(!is.null(subSQM$orfs$seqs))
+        { 
+        subSQM$orfs$seqs              = SQM$orfs$seqs   [orfs]
+        }
+    subSQM$orfs$tax                   = SQM$orfs$tax    [orfs    ,,drop=FALSE]
+    #    Taxonomy
+    if('tax16S' %in% names(subSQM$orfs))
+        {
+        subSQM$orfs$tax16S             = SQM$orfs$tax16S[orfs                ]
+        }
+    #    Markers
+    if('markers' %in% names(subSQM$orfs))
+        {
+        subSQM$orfs$markers           = SQM$orfs$markers [orfs]
+        }
+    ### Contigs
+    #    Table
     subSQM$contigs$table              = SQM$contigs$table[contigs,,drop=FALSE]
+    #    Abundances
     subSQM$contigs$abund              = SQM$contigs$abund[contigs,,drop=FALSE]
     subSQM$contigs$bases              = SQM$contigs$bases[contigs,,drop=FALSE]
-    subSQM$contigs$cov                = SQM$contigs$cov[contigs  ,,drop=FALSE]
-    subSQM$contigs$cpm                = SQM$contigs$cpm[contigs  ,,drop=FALSE]
-    subSQM$contigs$tpm                = SQM$contigs$tpm[contigs  ,,drop=FALSE]
-    subSQM$contigs$seqs               = SQM$contigs$seqs[contigs]
-    subSQM$contigs$tax                = SQM$contigs$tax[contigs  ,,drop=FALSE]
+    subSQM$contigs$cov                = SQM$contigs$cov  [contigs,,drop=FALSE]
+    subSQM$contigs$cpm                = SQM$contigs$cpm  [contigs,,drop=FALSE]
+    subSQM$contigs$tpm                = SQM$contigs$tpm  [contigs,,drop=FALSE]
+    #    Sequences
+    if(!is.null(subSQM$contigs$seqs))
+        {
+        subSQM$contigs$seqs           = SQM$contigs$seqs [contigs]
+        }
+    #    Taxonomy
+    subSQM$contigs$tax                = SQM$contigs$tax  [contigs,,drop=FALSE]
+    #    Binning info
     if('bins' %in% names(subSQM))
         {
-        subSQM$contigs$bins           = SQM$contigs$bins[contigs ,,drop=FALSE]
-        subSQM$bins$table             = SQM$bins$table[bins      ,,drop=FALSE]
-        x = aggregate(subSQM$contigs$abund, by=list(subSQM$contigs$bins[,1]), FUN=sum)
-        rownames(x)                   = x[,1]
-        x = x[rownames(subSQM$bin$table),-1,drop=FALSE]
-	nobin                         = colSums(subSQM$contigs$abund) - colSums(x)
-	if(sum(nobin)>0)              { x['No_bin',] = nobin }
-	subSQM$bins$abund             = as.matrix(x)
-	subSQM$bins$percent           = 100 * t(t(subSQM$bins$abund) / subSQM$total_reads)
-        
-	x = aggregate(subSQM$contigs$bases, by=list(subSQM$contigs$bins[,1]), FUN=sum)
-        rownames(x)                   = x[,1]
-        x = x[rownames(subSQM$bin$table),-1]
-        subSQM$bins$bases             = as.matrix(x)
-        
-	l = aggregate(subSQM$contigs$table$Length, by=list(subSQM$contigs$bins[,1]), FUN=sum)
-	n = l[,1]; l = l[,-1]; names(l) = n
-	l = l[rownames(subSQM$bin$table)]
-	subSQM$bins$length            = l
-	subSQM$bins$cov               = subSQM$bins$bases / subSQM$bins$length
-	subSQM$bins$cpm               = t(t(subSQM$bins$cov) / (subSQM$total_reads/1000000))
-        subSQM$bins$tax               = subSQM$bins$tax[bins     ,,drop=FALSE]
+	subSQM$contigs$bins           = SQM$contigs$bins[contigs ,,drop=FALSE]
+	### Bins
+	if(recalculate_bin_stats & (!('tax16S' %in% names(subSQM$orfs)) & 'markers' %in% names(subSQM$orfs)))
+            {
+            warning('You requested to recalculate bin stats but 16S or marker gene info are missing. Will not recalculate bin stats')
+            }
+	# Table and Taxonomy
+        if(recalculate_bin_stats & ('tax16S' %in% names(subSQM$orfs) & 'markers' %in% names(subSQM$orfs)))
+            {
+            bin_stats                 = get.bin.stats(subSQM)
+            subSQM$bins$table         = bin_stats[['table']]
+            subSQM$bins$tax           = bin_stats[['tax']] 
+        }else
+            {
+            subSQM$bins$table         = SQM$bins$table[bins,,drop=FALSE]
+            subSQM$bins$tax           = SQM$bins$tax  [bins,,drop=FALSE]
+            }
+	#    Abundances
+	bin_abunds                    = get.bin.abunds(subSQM)
+	subSQM$bins$abund             = bin_abunds[['abund']]
+	subSQM$bins$percent           = bin_abunds[['percent']]
+	subSQM$bins$bases             = bin_abunds[['bases']]
+	subSQM$bins$length            = bin_abunds[['length']]
+	subSQM$bins$cov               = bin_abunds[['cov']]
+	subSQM$bins$cpm               = bin_abunds[['cpm']]
         }
 
+    ### Taxonomy
     subSQM$taxa$superkingdom$abund    = aggregate_taxa(subSQM, 'superkingdom', tax_source)
     subSQM$taxa$phylum$abund          = aggregate_taxa(subSQM, 'phylum'      , tax_source)
     subSQM$taxa$class$abund           = aggregate_taxa(subSQM, 'class'       , tax_source)
@@ -347,6 +504,7 @@ subsetORFs = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = F
     subSQM$taxa$genus$percent         = 100 * t(t(subSQM$taxa$genus$abund)        / subSQM$total_reads)
     subSQM$taxa$species$percent       = 100 * t(t(subSQM$taxa$species$abund)      / subSQM$total_reads)
 
+    ### Functions
     if('KEGG' %in% names(subSQM$functions))
         {
         KEGG                          = aggregate_fun(subSQM, 'KEGG', trusted_functions_only, ignore_unclassified_functions)
@@ -415,42 +573,24 @@ subsetORFs = function(SQM, orfs, tax_source = 'orfs', trusted_functions_only = F
             }
     }
 
-    if(!is.null(subSQM$misc$RecA_cov))
+    ### COPY NUMBERS
+    subSQM$misc$single_copy_cov = get_median_single_copy_cov(subSQM)
+
+    if(!any(is.na(subSQM$misc$single_copy_cov)) & rescale_copy_number)
         {
-        if(rescale_copy_number)
+        subSQM$misc$single_copy_cov = get_median_single_copy_cov(subSQM)
+        for(method in names(subSQM$functions))
             {
-            if('COG0468' %in% rownames(COG$cov))
-                {
-                if(all(COG$cov['COG0468',]>0))
-	            {
-                    RecA = COG$cov['COG0468',]
-		}else
-	            {
-	            warning('RecA has zero abundance in at least one sample in this subset. Will not rescale copy numbers.')
-		    RecA = SQM$misc$RecA_cov
-		    }
-            }else
-                {
-                warning('RecA is not present in this subset. Will not rescale copy numbers.')
-                RecA = SQM$misc$RecA_cov
-                } 
-        }else
-           {
-           RecA = SQM$misc$RecA_cov
-           }
-        if('KEGG' %in% names(subSQM$functions)) { subSQM$functions$KEGG$copy_number = t(t(KEGG$cov) / RecA) }
-        if('COG'  %in% names(subSQM$functions)) { subSQM$functions$COG$copy_number  = t(t(COG$cov ) / RecA) }
-        if('PFAM' %in% names(subSQM$functions)) { subSQM$functions$PFAM$copy_number = t(t(PFAM$cov) / RecA) }
-        for(method in subSQM$misc$ext_annot_sources)
+            subSQM$functions[[method]]$copy_number = t(t(subSQM$functions[[method]]$cov) / subSQM$misc$single_copy_cov)
+            }
+    } else
+        {
+        for(method in names(subSQM$functions))
             {
-            subSQM$functions[[method]]$copy_number = t(t(ext_annots[[method]]$cov) / RecA)
+             subSQM$functions[[method]]$copy_number =  subSQM$functions[[method]]$copy_number[rownames(subSQM$functions[[method]]$abund),]
             }
-        subSQM$misc$RecA_cov              = RecA
         }
 
-
-    #subSQM$total_reads		     = colSums(subSQM$contigs$abund)
-
     return(subSQM)
     }
 
